@@ -1,3 +1,6 @@
+"""
+Utilities to read and write csv files
+"""
 from __future__ import annotations
 import csv as _csv
 import os as _os
@@ -16,7 +19,16 @@ def _as_number_if_possible(s, fallback=None, accept_fractions=True, accept_expon
     return n if n is not None else fallback
 
 
-def replace_non_alfa(s):
+def replace_non_alpha(s: str) -> str:
+    """
+    Remove any non-alphanumeric characters, replace spaces with _
+
+    Args:
+        s: the string to sanitize
+
+    Returns:
+        a copy of s with all non-alphanumeric characters removed
+    """
     # type: (str) -> str
     TRANSLATION_STRING = '\x00\x01\x02\x03\x04\x05\x06\x07\x08\t\n\x0b\x0c\r\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f !"#$%&\'__x+,__/0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~\x7f\x80\x81\x82\x83\x84\x85\x86\x87\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f\x90\x91\x92\x93\x94\x95\x96\x97\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xab\xac\xad\xae\xaf\xb0\xb1\xb2\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf\xd0\xd1\xd2\xd3\xd4\xd5\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7\xe8\xe9\xea\xeb\xec\xed\xee\xef\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff'
     s = s.translate(TRANSLATION_STRING)
@@ -25,9 +37,8 @@ def replace_non_alfa(s):
     return s
 
 
-def _normalize_column_name(name):
-    # type: (str) -> str
-    name = replace_non_alfa(name)
+def _normalize_column_name(name: str) -> str:
+    name = replace_non_alpha(name)
     if name and name[0] in '0123456789':
         name = 'n' + name
     name = name.strip().rstrip('_')
@@ -54,6 +65,12 @@ def _treat_duplicates(columns: Seq[str]) -> list[str]:
 def readcsv_numpy(csvfile: str) -> numpy.ndarray:
     """
     Read CSV into a numpy array
+
+    Args:
+        csvfile: the file to read
+
+    Returns:
+        the contents of the file as a 2D numpy array
     """
     return numpy.genfromtxt(csvfile, names=None, delimiter=',')
 
@@ -76,19 +93,24 @@ class _Rows(list):
 def readcsv(csvfile, columns:list[str]=None, asnumber=True,
             accept_exponential_numbers=False,
             typeconversions=None,
-            prefer_fractions=False, rowname='Row', dialect='excel'):
+            prefer_fractions=False, dialect='excel'
+            ) -> RecordList:
     """
-    read a CSV file into a namedtuple
+    Read a CSV file into a namedtuple
 
-    if the first collumn is all text: assume these are the column names
+    If the first collumn is all text, assume these are the column names
 
-    columns: a seq of column names, if the first row of data is not
-             a list header
-    rowname: override the row name specified in the CSV file (if any)
-    asnumber: convert strings to numbers if they can be converted
-    typeconversions: if given, a dict of the form {column:type}
-    prefer_fractions: If True, interpret expressions like 3/4 as Fractions,
-                      otherwise, as str. 
+    Args:
+        columns: a seq of column names, if the first row of data is not
+                 a list header
+        asnumber: convert strings to numbers if they can be converted
+        typeconversions: if given, a dict of the form {column:type}
+        accept_exponential_numbers: if True, parse a string 1.5e4 as a number
+        prefer_fractions: If True, interpret expressions like 3/4 as Fractions,
+            otherwise, as str. 
+    
+    Returns:
+        a RecordList
     """
     assert dialect in _csv.list_dialects()
     mode = "U"
@@ -117,8 +139,7 @@ def readcsv(csvfile, columns:list[str]=None, asnumber=True,
                             "Number-like cells found in the first-row")
     normalized_columns = [_normalize_column_name(col) for col in columns]
     columns = _treat_duplicates(normalized_columns)
-    rowname = rowname if rowname is not None else 'Row'
-    Row = _namedtuple(rowname, ' '.join(columns))
+    Row = _namedtuple('Row', ' '.join(columns))
     numcolumns = len(columns)
     rows = _Rows()
     for row in r:
@@ -168,7 +189,7 @@ def write_records_as_csv(records: list, outfile: str) -> None:
     f.close()
 
 
-def writecsv(rows: list, outfile:str, column_names:Seq[str]=None):
+def writecsv(rows: list, outfile:str, column_names:Seq[str]=None) -> None:
     """
     write a sequence of tuples/named tuples/dataclasses to outfile as CSV
 
