@@ -1,3 +1,6 @@
+"""
+More itertools
+"""
 from __future__ import annotations
 from itertools import *
 import operator as _operator
@@ -58,9 +61,12 @@ def quantify(iterable: Iter, pred: Callable=bool) -> int:
     
     Example
     =======
-    
-    >>> quantify("ABACA", lambda letter:letter=='A')
-    3
+
+    .. code::
+
+        >>> quantify("ABACA", lambda letter:letter=='A')
+        3
+
     """
     return sum(map(pred, iterable))
 
@@ -68,9 +74,12 @@ def quantify(iterable: Iter, pred: Callable=bool) -> int:
 def drop(seq: Iter[T], n: int) -> Iter[T]:
     """
     return an iterator over seq with n elements consumed
-    
-    >>> list(drop(range(10), 3))
-    [3, 4, 5, 6, 7, 8, 9]
+
+    Example
+    -------
+
+        >>> list(drop(range(10), 3))
+        [3, 4, 5, 6, 7, 8, 9]
     """
     return islice(seq, n, None)
 
@@ -98,6 +107,18 @@ def ncycles(seq: Iter[T], n: int) -> Iter[T]:
 
 
 def dotproduct(vec1: Iter[T], vec2: Iter[T]) -> T:
+    """
+    Returns the dot product (the sum of the product between each pair)
+    between vec1 and vec2
+
+    Args:
+        vec1: a seq. of T
+        vec2: a seq. of T
+
+    Returns:
+        the sum of the product of vec1_n * vec2_n for each n
+
+    """
     return sum(map(_operator.mul, vec1, vec2))
 
 
@@ -115,15 +136,15 @@ def repeatfunc(func, times=None, *args):
 
 def pairwise(iterable: Iter[T]) -> Iter[Tup[T, T]]:
     """
-    s -> (s0,s1), (s1,s2), (s2, s3), ..."
-    
-    similar to window(seq, size=2, step=1)
+    Similar to window(seq, size=2, step=1)
     
     Example
-    =======
-    
-    >>> list(pairwise(range(4)))
-    [(0, 1), (1, 2), (2, 3)]
+    -------
+
+    .. code::
+
+        >>> list(pairwise(range(4)))
+        [(0, 1), (1, 2), (2, 3)]
     """    
     a, b = tee(iterable)
     try:
@@ -200,11 +221,9 @@ def chunks(start:int, stop:int=None, step:int=None) -> Iter[Tup[int, int]]:
     Example
     =======
 
-    chunks(0, 10, 3)
-    (0, 3)
-    (3, 3)
-    (6, 3)
-    (9, 1)
+    >>> list(chunks(0, 10, 3))
+    [(0, 3), (3, 3), (6, 3), (9, 1)]
+
     """
     start, stop, step = _parse_range(start, stop, step)
     while start < stop:
@@ -214,6 +233,7 @@ def chunks(start:int, stop:int=None, step:int=None) -> Iter[Tup[int, int]]:
 
 
 def isiterable(obj, exclude=(str,)) -> bool:
+    """Returns True if obj is iterable"""
     if exclude:
         return hasattr(obj, '__iter__') and (not isinstance(obj, exclude))
     return hasattr(obj, '__iter__')
@@ -223,7 +243,8 @@ def grouper(seq: Iter, n: int, fillvalue=None) -> Iter:
     """
     Collect data into fixed-length chunks or blocks
     
-    >>> for group in grouper('ABCDEFG', 3, fillvalue='x'): print(group)
+    >>> for group in grouper('ABCDEFG', 3, fillvalue='x'):
+    ...     print(group)
     ('A', 'B', 'C')
     ('D', 'E', 'F')
     ('G', 'x', 'x')
@@ -252,10 +273,12 @@ def partialsum(seq: Iter[T], start=0) -> Iter[T]:
     """
     for each elem in seq return the partial sum
 
-    n0 -> n0
-    n1 -> n0 + n1
-    n2 -> n0 + n1 + n2
-    n3 -> n0 + n1 + n2 + n3
+    ::
+
+        n0 -> n0
+        n1 -> n0 + n1
+        n2 -> n0 + n1 + n2
+        n3 -> n0 + n1 + n2 + n3
     """
     accum = start
     for i in seq:
@@ -266,9 +289,11 @@ def partialmul(seq: Iter[T], start=1) -> Iter[T]:
     """
     return the accumulated multiplication
 
-    n0 -> n0
-    n1 -> n1 * n0
-    n2 -> n2 * n1 * n0
+    ::
+
+        n0 -> n0
+        n1 -> n1 * n0
+        n2 -> n2 * n1 * n0
     """
     assert isiterable(seq)
     accum = start
@@ -299,10 +324,11 @@ def ilen(seq: Iter) -> int:
 
 
 def avg(seq: Iter[T], empty=0) -> T:
-    """ Return the average of seq, or `empty` if the seq. is empty
+    """
+    Return the average of seq, or `empty` if the seq. is empty
 
-    NB: if you know the size of seq, it is faster to do
-        `sum(seq) / len_of_seq`
+    **NB**: if you know the size of seq, it is faster to do
+    `sum(seq) / len_of_seq`
 
     >>> avg(range(1_000_000))
     499999.5
@@ -319,9 +345,10 @@ def flatten(s: Iter[U[T, Iter[T]]], exclude=(str,), levels=inf) -> Iter[T]:
     return an iterator to the flattened items of sequence s
     strings are not flattened
 
-    seq = [1, [2, 3], [4, [5, 6]]]
-    list(flatten(seq))
-    -> [1, 2, 3, 4, 5, 6]
+    >>> seq = [1, [2, 3], [4, [5, 6]]]
+    >>> list(flatten(seq))
+    [1, 2, 3, 4, 5, 6]
+
     """
     for item in s:
         if isinstance(item, exclude or levels <= 0) or not hasattr(item, '__iter__'):
@@ -350,7 +377,16 @@ def flatdict(d: dict) -> list:
     return out
 
 
-def flattened(s: Iter[U[T, Iter[T]]], exclude=(str,), levels=inf, out=None) -> List[T]:
+def flattened(s: Iter[U[T, Iter[T]]], exclude=(str,), levels=inf, out:list=None) -> List[T]:
+    """
+    Like flatten, but returns a list instead of an iterator
+
+    Args:
+        s: the seq to flatten.
+        exclude: types to exclude
+        levels: how many levels to flatten
+        out: if given, the flattened result is appended to this list
+    """
     if out is None:
         out = []
     _flattened2(s, out, exclude, levels)
@@ -363,43 +399,25 @@ def _flattened2(s, out:list, exclude, levels:int) -> None:
             out.append(item)
         else:
             _flattened2(item, out, exclude, levels-1)
-        
 
 
-# as a reference, a non-recursive version. It is slower than flatten
-def _flatten_nonrec(iterable):
-    iterator, sentinel, stack = iter(iterable), object(), []
-    pop = stack.pop
-    append = stack.append
-    while True:
-        value = next(iterator, sentinel)
-        if value is sentinel:
-            if not stack:
-                break
-            iterator = pop()
-        elif isinstance(value, str):
-            yield value
-        else:
-            try:
-                new_iterator = iter(value)
-            except TypeError:
-                yield value
-            else:
-                append(iterator)
-                iterator = new_iterator
-
-            
 def flattenonly(l, types):
     """
     Flatten only if subsequences are of type 'type' 
 
-    type: a type or a tuple of types, as passed to isinstance
+    Args:
+        l: a seq
+        type: a type or a tuple of types, as passed to isinstance
 
-    Example: flatten only list items, pass tuples untouched
+    Example
+    -------
+
+    flatten only list items, pass tuples untouched
+
+    >>> l = [1, [2, 3], (4, 5)]
+    >>> list(flattenonyl(l, list))
+    [1, 2, 3, (4, 5)]
 
-    l = [1, [2, 3], (4, 5)]
-    list(flattenonyl(l, list))
-    --> [1, 2, 3, (4, 5)]
     """
     for elem in l:
         if isinstance(elem, types):
@@ -413,7 +431,7 @@ def zipflat(*seqs):
     like izip but flat. It has the same effect as flatten(zip(seqA, seqB), levels=1)
     
     Example
-    ~~~~~~~
+    -------
     
     >>> list(zipflat(range(5), "ABCDE"))
     [0, 'A', 1, 'B', 2, 'C', 3, 'D', 4, 'E']
@@ -423,8 +441,7 @@ def zipflat(*seqs):
             yield elem
 
             
-def butlast(seq):
-    # type: (Iter[T]) -> Iter[T]
+def butlast(seq: Iter[T]) -> Iter[T]:
     """
     iterate over seq[:-1]
     
@@ -438,8 +455,7 @@ def butlast(seq):
         lastitem = i
 
         
-def butn(seq, n):
-    # type: (Iter[T], int) -> Iter[T]
+def butn(seq: Iter[T], n: int) -> Iter[T]:
     """
     iterate over seq[:-n]
     """
@@ -451,28 +467,24 @@ def butn(seq, n):
         d.append(x)
 
         
-def intercalate(seq, item):
-    # type: (Iter[T], Any) -> Iter[T]
+def intercalate(seq: Iter[T], item) -> Iter[T]:
     """
+    Intercalate *item* between elements of *seq*
+
     Example
-    =======
-    
+    -------
+
     >>> list(intercalate(range(5), "X"))
     [0, 'X', 1, 'X', 2, 'X', 3, 'X', 4]
     """
     return butlast(zipflat(seq, repeat(item)))
 
 
-def apply_funcs(seq, *funcs):
-    tees = tee(seq, len(funcs))
-    return [func(t) for t, func in zip(tees, funcs)]
-
-
-def reductions(seq, func, start=0):
-    # type: (Iter[T], Callable, T) -> Iter[T]
+def reductions(seq: Iter[T], func: Callable, start=0) -> Iter[T]:
     """
-    return an iterator of the partial values of the reduction.
-    the last value is always equal to reduce(func, seq, start)
+    Return an iterator of the partial values of the reduction.
+
+    The last value is always equal to reduce(func, seq, start)
     """
     out = start
     it = iter(seq)
@@ -484,30 +496,33 @@ def reductions(seq, func, start=0):
             raise StopIteration
 
         
-def mesh(xs, ys):
-    # type: (Iter[T], Iter[T2]) -> Iter[Tup[T, T2]]
+def mesh(xs: Iter[T], ys: Iter[T2]) -> Iter[Tup[T, T2]]:
     """
     iterator over the lexicographical pairs
-    
-    (x1 y1) (x1 y2) (x1 y3) ... (x1 yn)
-    (x2 y1) (x2 y2) (x2 y3) ... (x2 yn)
-    ...
-    (xn y1) ...                 (xn yn)
-    
+
+    ::
+
+        (x1 y1) (x1 y2) (x1 y3) ... (x1 yn)
+        (x2 y1) (x2 y2) (x2 y3) ... (x2 yn)
+        ...
+        (xn y1) ...                 (xn yn)
+
     Example
     =======
-    
-    In [4]: list(iterlib.mesh((1,2,3), "A B C".split()))
-    Out[4]:
-    [(1, 'A'),
-     (1, 'B'),
-     (1, 'C'),
-     (2, 'A'),
-     (2, 'B'),
-     (2, 'C'),
-     (3, 'A'),
-     (3, 'B'),
-     (3, 'C')]
+
+    .. code::
+
+        In [4]: list(iterlib.mesh((1,2,3), "A B C".split()))
+        Out[4]:
+        [(1, 'A'),
+         (1, 'B'),
+         (1, 'C'),
+         (2, 'A'),
+         (2, 'B'),
+         (2, 'C'),
+         (3, 'A'),
+         (3, 'B'),
+         (3, 'C')]
     """
     for x in xs:
         for y in ys:
@@ -524,11 +539,11 @@ def mesh3(A, B, C):
                 yield (a, b, c)
 
                 
-def unique(seq):
-    # type: (Iter[T]) -> Set[T]
-    """ 
-    Return only unique elements of a sequence. If seq is not an iterator,
-    it is better to call set instead
+def unique(seq: Iter[T]) -> Set[T]:
+    """
+    Return only unique elements of a sequence.
+
+    If seq is not an iterator, it is better to call set instead
 
     >>> tuple(unique((1, 2, 3)))
     (1, 2, 3)
@@ -572,8 +587,10 @@ def interleave(seqs, pass_exceptions=()):
 
 def splitInChunks(s: Seq[T], chunksize: int) -> List[Seq[T]]:
     """
+    splits *s* into chunks
+
     Example
-    =======
+    -------
 
     >>> s = "FooBarBaz"
     >>> splitInChunks(s, 3)
@@ -587,4 +604,4 @@ if __name__ == '__main__':
     doctest.testmod()
 
 
-del Set, Any, List, TypeVar, Callable, U, Opt, Iter, Tup
+del Set, Any, List, TypeVar, Callable, U, Opt, Tup
